@@ -1,10 +1,14 @@
+# coding: utf-8
 from operator import attrgetter
 
-from django import template
+from django import template, VERSION as DJANGO_VERSION
 from django.conf import settings
 from django.http import Http404
 from django.utils.translation import ugettext_lazy as _
 
+from ..util import get_sort_field
+
+
 register = template.Library()
 
 DEFAULT_SORT_UP = getattr(settings, 'DEFAULT_SORT_UP', '&uarr;')
@@ -13,8 +17,8 @@ INVALID_FIELD_RAISES_404 = getattr(settings, 'SORTING_INVALID_FIELD_RAISES_404',
 
 sort_directions = {
     'asc': {'icon': DEFAULT_SORT_UP, 'inverse': 'desc'},
-    'desc': {'icon': DEFAULT_SORT_DOWN, 'inverse': 'asc'},
-    '': {'icon': DEFAULT_SORT_DOWN, 'inverse': 'asc'},
+    'desc': {'icon': DEFAULT_SORT_DOWN, 'inverse': ''},
+    '': {'icon': '', 'inverse': 'asc'},
 }
 
 
@@ -85,8 +89,12 @@ class SortAnchorNode(template.Node):
             getvars['dir'] = sortdir['inverse']
             icon = sortdir['icon']
         else:
+            getvars['dir'] = 'asc'
             icon = ''
 
+        if getvars['dir'] == '':
+            getvars.pop('dir', None)
+
         if len(getvars.keys()) > 0:
             urlappend = "&%s" % getvars.urlencode()
         else:
@@ -97,7 +105,10 @@ class SortAnchorNode(template.Node):
         else:
             title = self.title
 
-        url = '%s?sort=%s%s' % (request.path, self.field, urlappend)
+        if 'dir' in getvars:
+            url = '%s?sort=%s%s' % (request.path, self.field, urlappend)
+        else:
+            url = '%s%s%s' % (request.path, '?' if urlappend else '', urlappend)
         return '<a href="%s" title="%s">%s</a>' % (url, self.title, title)
 
 
@@ -133,7 +144,41 @@ class SortedDataNode(template.Node):
 
         # Python sorting if not a field
         field = ordering[1:] if ordering[0] == '-' else ordering
-        return field not in queryset.model._meta.get_all_field_names()
+
+        if DJANGO_VERSION < (1, 8):
+            field_names = queryset.model._meta.get_all_field_names()
+        else:
+            field_names = [f.name for f in queryset.model._meta.get_fields()]
+
+        return field not in field_names
+
+    def sort_queryset(self, queryset, ordering):
+        if not ordering:
+            return queryset
+
+        if queryset.exists():
+            if self.need_python_sorting(queryset, ordering):
+                # Fallback on pure Python sorting (much slower on large data)
+
+                # The field name can be prefixed by the minus sign and we need to
+                # extract this information if we want to sort on simple object
+                # attributes (non-model fields)
+                if ordering[0] == '-':
+                    if len(ordering) == 1:
+                        # Prefix without field name
+                        raise template.TemplateSyntaxError
+
+                    reverse = True
+                    name = ordering[1:]
+                else:
+                    reverse = False
+                    name = ordering
+                if hasattr(queryset[0], name):
+                    return sorted(queryset, key=attrgetter(name), reverse=reverse)
+                else:
+                    raise AttributeError()
+            else:
+                return queryset.order_by(ordering)
 
     def render(self, context):
         if self.context_var is not None:
@@ -142,36 +187,15 @@ class SortedDataNode(template.Node):
             key = self.queryset_var.var
 
         queryset = self.queryset_var.resolve(context)
-        ordering = context['request'].field
+        ordering = get_sort_field(context['request'])
 
-        if ordering:
-            try:
-                if self.need_python_sorting(queryset, ordering):
-                    # Fallback on pure Python sorting (much slower on large data)
-
-                    # The field name can be prefixed by the minus sign and we need to
-                    # extract this information if we want to sort on simple object
-                    # attributes (non-model fields)
-                    if ordering[0] == '-':
-                        if len(ordering) == 1:
-                            raise template.TemplateSyntaxError
-
-                        reverse = True
-                        name = ordering[1:]
-                    else:
-                        reverse = False
-                        name = ordering
-                    context[key] = sorted(queryset, key=attrgetter(name), reverse=reverse)
-                else:
-                    context[key] = queryset.order_by(ordering)
-            except template.TemplateSyntaxError:
-                # Seems spurious to me...
-                if INVALID_FIELD_RAISES_404:
-                    raise Http404(
-                        'Invalid field sorting. If DEBUG were set to '
-                        'False, an HTTP 404 page would have been shown instead.')
-                context[key] = queryset
-        else:
+        try:
+            context[key] = self.sort_queryset(queryset, ordering)
+        except (template.TemplateSyntaxError, AttributeError):
+            if INVALID_FIELD_RAISES_404:
+                raise Http404(
+                    'Invalid field sorting. If DEBUG were set to '
+                    'False, an HTTP 404 page would have been shown instead.')
             context[key] = queryset
 
         return u''
